/*
*  Rubrica
*  file: prefer.c
*
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gnome.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <gnome.h>
#include <gnome-xml/tree.h>
#include <gnome-xml/parser.h>

#include "prefer.h"
#include "dictionary.h"
#include "error.h"
#include "rubrica.h"

RubricaPreferences *pref=NULL;
GnomePropertyBox *pbox;
Dictionary user_dict;
GnomePropertyBox *propbox = NULL; 

gboolean load_pref = FALSE;
gboolean default_file_checked = FALSE;
gboolean language_change = FALSE;
gboolean language_code_change = FALSE;


void rubrica_preferences_init(gchar *dir, gchar *conf)
{  
  pref = (RubricaPreferences*) g_malloc(sizeof(RubricaPreferences));

  pref->show_path = FALSE;           /* FALSE - show filename, 
					TRUE  - show path/filename */
  pref->default_addressbook = TRUE;  /* TRUE  - load default addreesbook when start */
  pref->alpha = TRUE;                /* TRUE  - ordering is alphabetic */
  pref->date = FALSE;                /* TRUE  - oredering is by date (to do) */
  pref->increasing = TRUE;           /* TRUE  - show from minor to major */
  pref->confirm = TRUE;              /* TRUE  - confirm exit from rubrica */
  pref->backup = TRUE;               /* TRUE  - make backup */
  pref->zip = FALSE;                 /* TRUE  - gzip addressbook */

  pref->user_dict = FALSE;           /* TRUE  - use user defined dictionary */
  pref->use_english = FALSE;         /* TRUE  - english is default language
					FALSE - italian is default language */
  pref->trans_loaded = FALSE;        /* TRUE  - user dictionary has been loaded */

  pref->langcode = ITALIAN;          
  pref->language = g_strdup("Italian");
  pref->short_language = g_strdup("IT");

  pref->conf_filename = g_strdup(conf);

  pref->path = g_strdup(g_get_home_dir());  
  pref->addressbook_name = g_strdup("rubrica.rub");

  pref->user_dictionary = NULL;

  rubrica_preferences_save(conf);
}

gint rubrica_preferences_load()
{
  gchar *home_dir;
  gchar *fileconf;
  gchar *dirconf;
  
  home_dir = g_get_home_dir();  
  dirconf  = g_strdup_printf("%s/.rubrica", home_dir);
  fileconf = g_strdup_printf("%s/.rubrica/rubrica.conf", home_dir);
 
  if (!g_file_exists(fileconf))
    /* first time that rubrica is run, or config file was deleted/moved */
    {
      g_print("\nrubrica initialization...");      

      if (!g_file_exists(dirconf))
	mkdir(dirconf, S_IRUSR | S_IWUSR | S_IXUSR);
      
      rubrica_preferences_init(dirconf, fileconf);

      return 0;
    }
  else    /* load preferences from configuration file */
    {
      gboolean err;

      /* need memory */
      pref = (RubricaPreferences*) g_malloc(sizeof(RubricaPreferences));
      
      if (pref == NULL)
	{
	  g_error("Out of memory");
	  exit;
	}

      /*	load preferences
      */
      err = rubrica_load_pref(fileconf);  

      if (err)      /* can't read the configuration file */
	{
	  g_warning("Can't read configuration file. Rubrica will use default option");
	  g_free(pref);

	  rubrica_preferences_init(dirconf, fileconf);
	}

      /*	if user want use his dictionary
       */
      if (pref->user_dict)
	{	  
	  gchar *sh_lang;

	  err = rubrica_load_user_dictionary(pref->language, &sh_lang);
	  
	  if (!err)    /* user dictionary has been readed */
	    {
	      pref->trans_loaded = TRUE; 
	      g_print(_("User dictionary: %s\n"), pref->user_dictionary);
	    }
	  /* error messages have been showed from rubrica_load_user_dictionary */
	}    
    }
  load_pref = TRUE;
  g_print("Dictionary loaded...");

  g_free(dirconf);
  g_free(fileconf);
  
  return 1;
}


gboolean rubrica_load_pref(gchar *file)
{
  xmlDocPtr doc;
  
  doc = xmlParseFile(file);
  
  if (!doc || !doc->root || 
      !doc->root->name || 
      (g_strcasecmp(doc->root->name, "Rubrica_Configuration_File") != 0))
    return TRUE;    
  else
    { 
      /* 
	 here I read the configuration file 
      */ 
      gchar *buffer;
      xmlNodePtr nodo = doc->root->childs; 
      
      if (g_strcasecmp(nodo->name, "Config_data") != 0)
	  return TRUE;
      
      nodo = nodo->childs;

      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->show_path = TRUE;
      else 
	pref->show_path = FALSE; 
      nodo = nodo->next;
   
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->default_addressbook = TRUE;
      else 
	pref->default_addressbook = FALSE; 
      nodo = nodo->next;
        
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->alpha = TRUE;
      else 
	pref->alpha = FALSE; 
      nodo = nodo->next;
                
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->date = TRUE;
      else 
	pref->date = FALSE; 
      nodo = nodo->next;
              
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->increasing = TRUE;
      else 
	pref->increasing = FALSE; 
      nodo = nodo->next;
               
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->confirm = TRUE;
      else 
	pref->confirm = FALSE; 
      nodo = nodo->next;
              
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->backup = TRUE;
      else 
	pref->backup = FALSE; 
      nodo = nodo->next;
              
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->zip = TRUE;
      else 
	pref->zip = FALSE; 
      nodo = nodo->next;
              
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->user_dict = TRUE;
      else 
	pref->user_dict = FALSE; 
      nodo = nodo->next;
           
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->use_english = TRUE;
      else 
	pref->use_english = FALSE; 
      nodo = nodo->next;
                           
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "TRUE") == 0)
	pref->trans_loaded = TRUE;
      else 
	pref->trans_loaded = FALSE; 
      nodo = nodo->next;
              
      buffer = xmlNodeGetContent(nodo);
      if (g_strcasecmp(buffer, "ITALIAN") == 0)
	pref->langcode = ITALIAN;
      else 
	if (g_strcasecmp(buffer, "ENGLISH") == 0)
	  pref->langcode = ENGLISH; 
	else
	  pref->langcode = OTHER;
      nodo = nodo->next;
              
      pref->language = g_strdup(xmlNodeGetContent(nodo));
      nodo = nodo->next;
              
      pref->short_language = g_strdup(xmlNodeGetContent(nodo));
      nodo = nodo->next; 
        
      pref->conf_filename = g_strdup(xmlNodeGetContent(nodo));
      nodo = nodo->next; 
         
      pref->path = g_strdup(xmlNodeGetContent(nodo));
      nodo = nodo->next;       
             
      pref->addressbook_name = g_strdup(xmlNodeGetContent(nodo));
      nodo = nodo->next;      
              
      pref->user_dictionary = g_strdup(xmlNodeGetContent(nodo));
    }
  xmlFreeDoc(doc);  
    
  return FALSE;
}

void rubrica_save_preferences()
{
  gchar *filename;
  gchar *tmp;

  tmp = g_get_home_dir();  
  filename = g_strdup_printf("%s/.rubrica/rubrica.conf", tmp);
  rubrica_preferences_save(filename);
  
  g_free(filename);
}

void rubrica_preferences_save(gchar *filename)
{
  xmlDocPtr doc;
  xmlNodePtr newxml;
  xmlNodePtr nodo;

  doc = xmlNewDoc("1.0");

  doc->root = xmlNewDocNode(doc, NULL, "Rubrica_Configuration_File", NULL);

  newxml = xmlNewChild(doc->root, NULL, "Config_data", NULL);
        
  if (pref->show_path)
    nodo = xmlNewTextChild(newxml, NULL, "ShowFileName", "TRUE");
  else
    nodo = xmlNewTextChild(newxml, NULL, "ShowFileName", "FALSE");
 
  if(pref->default_addressbook)
    nodo = xmlNewTextChild(newxml, NULL, "UseDefaultAddressBook", "TRUE");  
  else
    nodo = xmlNewTextChild(newxml, NULL, "UseDefaultAddressBook", "FALSE");  
    
  if (pref->alpha)
    nodo = xmlNewTextChild(newxml, NULL, "AlphabeticOrdering", "TRUE");  
  else
    nodo = xmlNewTextChild(newxml, NULL, "AlphabeticOrdering", "FALSE");  
   
  if (pref->date)
    nodo = xmlNewTextChild(newxml, NULL, "DateOrdering", "TRUE");  
  else
    nodo = xmlNewTextChild(newxml, NULL, "DateOrdering", "FALSE");  
   
  if (pref->increasing)
    nodo = xmlNewTextChild(newxml, NULL, "ShowIncreasing", "TRUE");  
  else
    nodo = xmlNewTextChild(newxml, NULL, "ShowIncreasing", "FALSE");  
   
  if (pref->confirm)
    nodo = xmlNewTextChild(newxml, NULL, "ConfirmExit", "TRUE");  
  else
    nodo = xmlNewTextChild(newxml, NULL, "ConfirmExit", "FALSE");  
   
  if (pref->backup)
    nodo = xmlNewTextChild(newxml, NULL, "MakeBackup", "TRUE");  
  else
    nodo = xmlNewTextChild(newxml, NULL, "MakeBackup", "FALSE");  
   
  if (pref->zip)
    nodo = xmlNewTextChild(newxml, NULL, "ZipAddressBook", "TRUE");  
  else
    nodo = xmlNewTextChild(newxml, NULL, "ZipAddressBook", "FALSE");  
  
  if (pref->user_dict)
    nodo = xmlNewTextChild(newxml, NULL, "UserDictioanry", "TRUE");  
  else
    nodo = xmlNewTextChild(newxml, NULL, "UserDictionary", "FALSE");  
 
  if (pref->use_english)
    nodo = xmlNewTextChild(newxml, NULL, "UseEnglish", "TRUE");  
  else
    nodo = xmlNewTextChild(newxml, NULL, "UseEnglish", "FALSE");  
      
  if (pref->trans_loaded)
    nodo = xmlNewTextChild(newxml, NULL, "LoadTranslation", "TRUE");  
  else
    nodo = xmlNewTextChild(newxml, NULL, "LoadTranslation", "FALSE");  
   
  switch(pref->langcode)
    {
    case  ITALIAN:
      nodo = xmlNewTextChild(newxml, NULL, "LanguageCode", "ITALIAN");  
      break;
    case ENGLISH:
      nodo = xmlNewTextChild(newxml, NULL, "LanguageCode", "ENGLISH");  
      break;
    case OTHER:
      nodo = xmlNewTextChild(newxml, NULL, "LanguageCode", "OTHER");  
      break;
    default:
      break;
    }
   
  nodo = xmlNewTextChild(newxml, NULL, "Language", pref->language);  
    
  nodo = xmlNewTextChild(newxml, NULL, "ShortLanguage", pref->short_language); 
    
  nodo = xmlNewTextChild(newxml, NULL, "ConfFileName", pref->conf_filename); 
    
  nodo = xmlNewTextChild(newxml, NULL, "Path", pref->path); 
      
  nodo = xmlNewTextChild(newxml, NULL, "AddressBookName", 
			 pref->addressbook_name);  
    
  nodo = xmlNewTextChild(newxml, NULL, "UserDictionaryName", 
			 pref->user_dictionary);  
 
  if (xmlSaveFile(filename, doc) == -1) 
    {
      xmlFreeDoc(doc);
      
      g_warning("I can't save configuration file");     
    }
  
  xmlFreeDoc(doc);    
}

/*    functions that manages pref structure
 */
RubricaPreferences *rubrica_get_preferences()
{
  return (RubricaPreferences *) pref;
}

/*      get - set show_path
*/
void rubrica_pref_set_show_path(gboolean show_path)
{
  pref->show_path = show_path;
}

gboolean rubrica_pref_get_show_path(void)
{
  return (gboolean) pref->show_path;
}


/*      get - set address book
*/
void rubrica_pref_set_default_addressbook(gboolean default_ab)
{
  pref->default_addressbook = default_ab;
}

gboolean rubrica_pref_get_default_addressbook(void)
{
  return (gboolean) pref->default_addressbook;
}

/*      get - set alpha
 */
void rubrica_pref_set_alpha_order(gboolean ord)
{
  if (ord)
    {
      pref->alpha = TRUE;
      pref->date = FALSE;
    }
  else
    {
      pref->alpha = FALSE;
      pref->date = TRUE;      
    }
}

gboolean rubrica_pref_get_alpha_order(void)
{
  return (gboolean) pref->alpha;
}

/*      get - set date
 */
void rubrica_pref_set_date_order(gboolean ord)
{
  if (ord)
    {
      pref->alpha = FALSE;
      pref->date = TRUE;
    }
  else
    {
      pref->alpha = TRUE;
      pref->date = FALSE;      
    }  
}

gboolean rubrica_pref_get_data_order(void)
{
  return (gboolean) pref->date;
}

/*      get - set exit confirm
 */
void rubrica_pref_set_exit_confirm(gboolean confirm)
{
  pref->confirm = confirm;
}

gboolean rubrica_pref_get_exit_confirm(void)
{
  return (gboolean) pref->confirm;
}

/*      get - set increasing
 */
void rubrica_pref_set_increasing(gboolean inc)
{
  pref->increasing = (gboolean) inc;  /* true  show cards from minor to major 
					 false show cards from major to minor */
                                       
}

gboolean rubrica_pref_get_increasing(void)
{
  return (gboolean) pref->increasing;
}

/*      get - set do backup
 */
void rubrica_pref_set_do_backup(gboolean bk)
{
  pref->backup = bk;
}

gboolean rubrica_pref_get_do_backup(void)
{
  return (gboolean) pref->backup;
}

/*      get - set zip addressbook
 */
void rubrica_pref_set_zip_addressbook(gboolean zip)
{
  pref->zip = zip;
}

gboolean rubrica_pref_get_zip_addressbook(void)
{
  return (gboolean) pref->zip;
}

/*      get - set user dictionary
 */
gboolean rubrica_pref_get_use_user_dictionary(void)
{
  return (gboolean) pref->user_dict;
}

void rubrica_pref_set_user_dictionary(gboolean use)
{
  pref->user_dict = use;
}

/*      get - set use italian
 */
gboolean rubrica_pref_get_use_english(void)
{
  return (gboolean) pref->use_english;
}

void rubrica_pref_set_use_english(gboolean flag)
{
  pref->use_english = flag;
}


/*    have been dictionary loaded?
 */
gboolean rubrica_dictionary_is_loaded()
{
  return (gboolean) pref->trans_loaded;
}

/*      get - set language code
 */
LANG rubrica_pref_get_lang_code(void)
{
  return (LANG) pref->langcode;
}

void rubrica_pref_set_lang_code(LANG code)
{
  pref->langcode = code;
}

/*      get - set language
 */
gchar* rubrica_pref_get_language(void)
{
  return (gchar*) pref->language;
}

void rubrica_pref_set_language(gchar* lan)
{
  if (pref->language)
    g_free(pref->language);

  if (lan)
    pref->language = g_strdup(lan);
  else
    pref->language = NULL;
}

/*      get - set short language
 */
void rubrica_pref_set_short_language(gchar* sl)
{
  if (pref->short_language)
    g_free(pref->short_language);
      
  if (sl)
    pref->short_language = g_strdup(sl);
  else 
    pref->short_language = NULL;
}

gchar *rubrica_perf_get_short_language(void)
{
  return (gchar*) pref->short_language;
}

/*    get - set addressbook path (user defined) 
*/
void rubrica_pref_set_path(gchar *path)
{
  if (pref->path)
    g_free(pref->path);

  if (path)
    pref->path = g_strdup(path);
  else 
    pref->path = NULL; 

/*  rubrica_vista_set_label_path(pref->path); */
}

gchar *rubrica_pref_get_path(void)
{
  return (gchar*) pref->path;
}

/*    get -set addressbook name (user defined) 
*/
void rubrica_pref_set_addressbook_fname(gchar *name)
{
  if (pref->addressbook_name)
    g_free(pref->addressbook_name);

  if (name)
    pref->addressbook_name = g_strdup(name);  
  else 
    pref->addressbook_name = NULL; 

/*  rubrica_vista_set_label_file(pref->addressbook_name); */
}

gchar *rubrica_pref_get_addressbook_fname(void)
{
  return (gchar*) pref->addressbook_name;
}

/*      get - set dictionary name
*/
void rubrica_pref_set_dictionary_fname(gchar *name)
{
  if (pref->user_dictionary)
    g_free(pref->user_dictionary);

  if (name)
    pref->user_dictionary = g_strdup(name);    
  else 
    pref->user_dictionary = NULL;
}

gchar *rubrica_pref_get_dictionary_fname(void)
{
  return (gchar*) pref->user_dictionary;
}

/*
         ***************************************** 
*/
void preferences_set_pref_loaded(gboolean load)
{
  load_pref = load;
}

gboolean preferences_loaded(void)
{
  return (gboolean) load_pref;
}

void rubrica_pref_set_def_file_check(gboolean state)
{
  default_file_checked = state;
}

gboolean rubrica_pref_get_def_file_check(void)
{
  return (gboolean) default_file_checked;
}

void rubrica_set_prefbox(GtkWidget *w1, GtkWidget *w2, gboolean flag)
{
  if (flag)
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), TRUE);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w2), FALSE);
    }
  else
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w1), FALSE); 
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w2), TRUE);
    }
}

void rubrica_pref_lang_changed(gboolean state)
{
  language_change = state;
}

void rubrica_pref_lang_code_changed(gboolean state)
{
  language_code_change = state;
}

gboolean rubrica_pref_lang_is_changed()
{
  return language_change;
}

gboolean rubrica_pref_lang_code_is_changed()
{
  return language_code_change;
}


void rubrica_pref_button_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
  /*          user_data is a pointer to property box 
  */
  gboolean state;  
  GtkWidget *wid;
  
  wid = gtk_object_get_data(GTK_OBJECT(user_data), "fileentry_def_addbook");
  
  state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(togglebutton));  
  rubrica_pref_set_default_addressbook(state);
  
  if (state)
    gtk_widget_set_sensitive(GTK_WIDGET(wid), TRUE);              
  else
    {
      rubrica_pref_set_path(NULL);     
      rubrica_pref_set_addressbook_fname(NULL);
      gtk_widget_set_sensitive(GTK_WIDGET(wid), FALSE);      
    }
}

void rubrica_refresh_files(gchar *data)
{
  gchar *path=NULL, *file=NULL;
  
  path = g_dirname(data);
  file = g_basename(data);
  
  rubrica_pref_set_addressbook_fname(file);
  rubrica_pref_set_path(path);

  g_free(path);
}


/*
  extract preferences from struct and
  fill propertybox's field. 
*/
void rubrica_preferences_set(GtkWidget *prop, RubricaPreferences *p)
{
  GtkWidget *wid1, *wid2, *wid3;

  /*    show path, show filename
   */
  wid1 = gtk_object_get_data(GTK_OBJECT(prop), "radio_show_filename");
  wid2 = gtk_object_get_data(GTK_OBJECT(prop), "radio_show_path");
  if (p->show_path)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid2), TRUE);
  else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid1), TRUE);

  /*    load default addressbook when start
   */
  wid1 = gtk_object_get_data(GTK_OBJECT(prop), "check_def_addbook");
  wid2 = gtk_object_get_data(GTK_OBJECT(prop), "fileentry_def_addbook"); 
  wid3 = gtk_object_get_data(GTK_OBJECT(prop), "entry_filename"); 

  if (p->default_addressbook)
    { 
      /* user want load a default addressbook when rubrica start */
      gchar *str;

      /* extract path and addressbook filename and show them in entry widget */
      str = g_strdup_printf("%s/%s",p->path, p->addressbook_name);

      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid1), TRUE);
      gtk_entry_set_text(GTK_ENTRY(wid3), str);        
      gtk_widget_set_sensitive(GTK_WIDGET(wid2), TRUE);  

      g_free(str);
    }
  else 
    {
      /* not load addressbook at rubrica start */
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid1), FALSE);
      gtk_entry_set_text(GTK_ENTRY(wid3), "");             /* clear entry */
      gtk_widget_set_sensitive(GTK_WIDGET(wid2), FALSE);                  
    }

  /*    sort by: date, alphabetic
   */
  wid1 = gtk_object_get_data(GTK_OBJECT(prop), "radio_date");
  wid2 = gtk_object_get_data(GTK_OBJECT(prop), "radio_alpha");
  if (p->alpha)
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid2), TRUE);
  else
    if (p->date)
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid1), TRUE);

  /*    sort: increasing, decreasing
   */
  wid1 = gtk_object_get_data(GTK_OBJECT(prop), "radio_dec");
  wid2 = gtk_object_get_data(GTK_OBJECT(prop), "radio_inc");
  if (p->increasing)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid2), TRUE);
  else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid1), TRUE);

  /*    make backup: never, always
   */
  wid1 = gtk_object_get_data(GTK_OBJECT(prop), "radio_never");
  wid2 = gtk_object_get_data(GTK_OBJECT(prop), "radio_always");
  if (p->backup)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid2), TRUE);
  else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid1), TRUE);

  /*    confirm exit: never, always
   */
  wid1 = gtk_object_get_data(GTK_OBJECT(prop), "radio_confirm_no");
  wid2 = gtk_object_get_data(GTK_OBJECT(prop), "radio_confirm_yes");
  if (p->confirm)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid2), TRUE);
  else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid1), TRUE);

  /*    zip addressbook: yes, no    
   */
  wid1 = gtk_object_get_data(GTK_OBJECT(prop), "radio_zip_no");
  wid2 = gtk_object_get_data(GTK_OBJECT(prop), "radio_zip_yes");
  if (p->zip)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid2), TRUE);
  else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid1), TRUE);

  
  /*    use user dictionary: yes, no	
	set default language to
	- user language
	- english
	- italian
  */
  wid1 = gtk_object_get_data(GTK_OBJECT(prop), "radio_other_lang");
  wid2 = gtk_object_get_data(GTK_OBJECT(prop), "radio_def_lang");
  if (p->user_dict)
    {
      GtkWidget *wid3, *wid4, *wid5, *wid6;

      /*    set user language and language code
       */
      wid3 = gtk_object_get_data(GTK_OBJECT(prop), "entry_user_lang");
      wid4 = gtk_object_get_data(GTK_OBJECT(prop), "entry_user_lang_code");
      wid5 = gtk_object_get_data(GTK_OBJECT(prop), "radio_def_ita");
      wid6 = gtk_object_get_data(GTK_OBJECT(prop), "radio_def_eng");

      gtk_entry_set_text(GTK_ENTRY(wid3), p->language);
      gtk_entry_set_text(GTK_ENTRY(wid4), p->short_language);

      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid1), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(wid5), FALSE);
      gtk_widget_set_sensitive(GTK_WIDGET(wid6), FALSE);

      /* widgets that are related to this radio button, will be 
	 activated in the callback called when radio button 
	 receive the toggle signal. 
      */
    }
  else
    {
      GtkWidget *wid3, *wid4, *wid5, *wid6;

      /*    clear language and language code
       */
      wid3 = gtk_object_get_data(GTK_OBJECT(prop), "entry_user_lang");
      wid4 = gtk_object_get_data(GTK_OBJECT(prop), "entry_user_lang_code");

      gtk_entry_set_text(GTK_ENTRY(wid3), "");
      gtk_entry_set_text(GTK_ENTRY(wid4), "");

      /*	set default language to italian or english
       */
      wid5 = gtk_object_get_data(GTK_OBJECT(prop), "radio_def_ita");
      wid6 = gtk_object_get_data(GTK_OBJECT(prop), "radio_def_eng");
      
      gtk_widget_set_sensitive(GTK_WIDGET(wid5), TRUE);
      gtk_widget_set_sensitive(GTK_WIDGET(wid6), TRUE);
      if (p->use_english)
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid6), TRUE);
      else
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid5), TRUE);

      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid2), TRUE);
    }
}
